<template>
  <div class="trace-plot">
    <div class="filter-bar">
      <el-date-picker
        size="small"
        v-model="searchTime"
        @change="searchData"
        type="datetimerange"
        start-placeholder="开始日期"
        end-placeholder="结束日期"
        value-format="yyyy-MM-dd HH:mm:ss"
        :default-time="['00:00:00','23:59:59']"
      ></el-date-picker>
      <el-button @click="searchData" size="small" type="primary" class="btn-search">查 询</el-button>
    </div>
    <div class="check-target" :style="{height:fixedH+'px'}">
      <div class="controller">
        <!-- <el-checkbox v-model="isHideAll" @change="hideAllChange">隐藏全部</el-checkbox> -->
        <el-checkbox v-model="isInvert" @change="invertChange">反选</el-checkbox>
      </div>
      <div class="target-list">
        <div
          class="list-item"
          v-for="item in actObjs"
          :key="item.id"
          :class="{'deact':item.isHide}"
          :style="{borderColor:item.isHide?'#ccc':item.color}"
          @click="checkTarget(item)"
        >
          <!-- <img src alt :style="{backgroundColor:item.color}"/> -->
        </div>
      </div>
    </div>
    <!-- <canvas
      ref="trackArea"
      :width="cW"
      :height="cH"
      :style="{backgroundImage:`url(${panoramaPath})`}"
    ></canvas>-->
    <canvas
      ref="trackArea"
      :width="fixedW"
      :height="fixedH"
      :style="{backgroundImage:`url(${panoramaPath})`,backgroundSize:`${bgW}px ${bgH}px`}"
    ></canvas>
  </div>
</template>

<script>
import { getSearchList } from '@/api/search';
import { getPanoramaPic } from "@/api/panorama";
export default {
  data () {
    return {
      cW: 0,
      cH: 0,
      panoramaPath: '',
      trackData: [],
      activeObjHashMap: {},
      actObj: {},
      actObjs: [],
      searchTime: [
        this.$moment().format("YYYY-MM-DD 00:00:00"),
        this.$moment().format("YYYY-MM-DD 23:59:59")
        //this.$moment().format("YYYY-MM-DD HH:mm:ss")
      ],
      timer: null,
      colorArr: ['#F4DA40', '#0092BC', '#97D700', '#D0006F', '#D86018', '#653279', '#A45A2A', '#004B87', '#008C95', '#AA0061'],
      searchStartTimeStamp: 0,
      searchEndTimeStamp: 0,
      fixedW: 960,
      fixedH: 700,
      bgW: 0,
      bgH: 0,
      targetChecked: [],
      isHideAll: false,
      isInvert: false,
    }
  },
  mounted () {
    this.searchData();
    this.getPanorama();
    this.timer = setInterval(() => {
      this.searchData();
    }, 7000);
  },
  beforeDestroy () {
    console.log('beforeDestroy')
    clearInterval(this.timer);
  },
  methods: {
    hideAllChange(){
      this.actObjs.forEach(target=>{
        target.isHide = this.isHideAll;
      });
      this.drawTracePath();
    },
    invertChange(){
      console.log(this.isInvert)
      this.actObjs.forEach(target=>{
        target.isHide = !target.isHide;
      });
      let showOne = this.actObjs.find(target=>!target.isHide);
      if(!showOne){
        this.isHideAll = true;
      }else{
        this.isHideAll = false;
      }
      this.drawTracePath();
    },
    checkTarget (item) {
      item.isHide = !item.isHide;
      this.isHideAll = item.isHide;
      this.drawTracePath();
    },
    getPanorama () {
      let _this = this;
      getPanoramaPic().then(res => {
        let { panoramaPath, width, height } = res.data;
        _this.panoramaPath = panoramaPath + '?' + Math.random();
        _this.cW = width;
        _this.cH = height;
        //判断长宽比
        let ratio = res.data.width / res.data.height;
        if (ratio > (_this.fixedW / _this.fixedH)) {
          _this.bgW = _this.fixedW;
          _this.bgH = _this.bgW * res.data.height / res.data.width;
        } else {
          _this.bgH = _this.fixedH;
          _this.bgW = res.data.width * _this.bgH / res.data.height;
        }
      })
    },
    drawTracePath () {
      let canvas = this.$refs['trackArea'];
      let ctx = canvas.getContext('2d');
      ctx.clearRect(0, 0, canvas.width, canvas.height);
      //ctx.strokeStyle = 'yellow';
      //ctx.fillStyle = 'aqua';
      //ctx.globalAlpha=0.5;
      ctx.lineWidth = 3;
      //ctx.lineJoin='round';
      ctx.lineCap = 'round';
      this.actObjs.forEach(target => {
        if (!target.isHide) {
          ctx.beginPath();
          ctx.fillStyle = target.color;
          ctx.arc(target.posInfo[0].globalCoordX, target.posInfo[0].globalCoordY, 3, 0, Math.PI * 2);
          ctx.fill();
          for (var i = 1; i < target.posInfo.length; i++) {
            ctx.beginPath();
            ctx.arc(target.posInfo[i].globalCoordX, target.posInfo[i].globalCoordY, 3, 0, Math.PI * 2);
            ctx.fill();
          }
        }

      });
      this.actObjs.forEach(target => {
        if (!target.isHide) {
          ctx.beginPath();
          ctx.strokeStyle = target.color;
          ctx.moveTo(target.posInfo[0].globalCoordX, target.posInfo[0].globalCoordY);
          for (var i = 1; i < target.posInfo.length; i++) {
            ctx.lineTo(target.posInfo[i].globalCoordX, target.posInfo[i].globalCoordY)
          }
          ctx.stroke();
        }
      })
      // for (var target in this.actObj) {
      //   ctx.beginPath();
      //   //ctx.fillStyle = `rgb(${Math.floor(Math.random() * 55 + 150)},${Math.floor(Math.random() * 55 + 150)},${Math.floor(Math.random() * 55 + 150)})`;
      //   ctx.fillStyle = this.actObj[target][0].color;
      //   ctx.arc(this.actObj[target][0].globalCoordX, this.actObj[target][0].globalCoordY, 3, 0, Math.PI * 2);
      //   ctx.fill();
      //   for (var i = 1; i < this.actObj[target].length; i++) {
      //     ctx.beginPath();
      //     ctx.arc(this.actObj[target][i].globalCoordX, this.actObj[target][i].globalCoordY, 3, 0, Math.PI * 2);
      //     ctx.fill()
      //   }
      // }
      // for (var target in this.actObj) {
      //   ctx.beginPath();
      //   //ctx.strokeStyle = `rgb(${Math.floor(Math.random() * 20 + 220)},${Math.floor(Math.random() * 20 + 210)},${Math.floor(Math.random() * 55 + 200)})`;
      //   ctx.strokeStyle = this.actObj[target][0].color;
      //   ctx.moveTo(this.actObj[target][0].globalCoordX, this.actObj[target][0].globalCoordY);
      //   for (var i = 1; i < this.actObj[target].length; i++) {
      //     ctx.lineTo(this.actObj[target][i].globalCoordX, this.actObj[target][i].globalCoordY);
      //   }
      //   ctx.stroke();
      // }
    },
    searchData () {
      let _this = this;
      var param = {
        page: 1,
        size: 5000,
        searchTime: this.searchTime,
        alarmlevel: [],
        inputValue: '',
        tabs: [],
        tasks: [],
        treeNodes: [],
        isAll: false
      };
      getSearchList(param).then(res => {
        let filterArr = [];
        if (!res.data.datalist) {
          _this.activeObjHashMap = {};
          _this.actObj = {};
          _this.actObjs = [];
        }
        for (var key in _this.activeObjHashMap) {
          let innerKey = res.data.datalist && res.data.datalist.find(obj => obj.activeObject.id == key);
          if (!innerKey) {
            delete _this.activeObjHashMap[key];
            delete _this.actObj[key];
            let index = _this.actObjs.findIndex(one => one.id == innerKey);
            _this.actObjs.splice(index, 1)
          }
        }

        res.data.datalist && res.data.datalist.forEach(obj => {
          if (_this.activeObjHashMap[obj.activeObject.id] !== 1) {
            _this.activeObjHashMap[obj.activeObject.id] = 1
            filterArr.push(obj)
          }
        });

        filterArr.forEach(item => {
          item.activeObject.targetInfo.forEach(target => {
            if (target.targetType == 'UniquelID') {
              let attribute = JSON.parse(target.attribute);
              let someone = _this.actObjs.find(obj => obj.id == target.targetId);
              if (someone) {
                someone.posInfo.unshift({ globalCoordX: attribute.globalCoordX, globalCoordY: attribute.globalCoordY, timeStamp: Date.parse(item.activeObject.picDate) });
              } else {
                _this.actObjs.push({ id: target.targetId, picSmUrl: target.picSmUrl, posInfo: [{ globalCoordX: attribute.globalCoordX, globalCoordY: attribute.globalCoordY, timeStamp: Date.parse(item.activeObject.picDate) }] });
              }

              if (_this.actObj[target.targetId]) {
                _this.actObj[target.targetId].unshift({ picSmUrl: target.picSmUrl, globalCoordX: attribute.globalCoordX, globalCoordY: attribute.globalCoordY, timeStamp: Date.parse(item.activeObject.picDate) });

              } else {
                _this.actObj[target.targetId] = [{ picSmUrl: target.picSmUrl, globalCoordX: attribute.globalCoordX, globalCoordY: attribute.globalCoordY, timeStamp: Date.parse(item.activeObject.picDate) }];

              }
            }
          })
        });

        _this.searchStartTimeStamp = Date.parse(_this.searchTime[0]);
        _this.searchEndTimeStamp = Date.parse(_this.searchTime[1]);

        for (var target in _this.actObj) {
          _this.actObj[target] = _this.actObj[target].filter(item =>
            item.timeStamp >= _this.searchStartTimeStamp && item.timeStamp <= _this.searchEndTimeStamp
          )
          if (_this.actObj[target].length == 0) {
            delete _this.actObj[target]
          }
        }

        for (var i = 0; i < _this.actObjs.length; i++) {
          _this.actObjs[i].posInfo = _this.actObjs[i].posInfo.filter(item => item.timeStamp >= _this.searchStartTimeStamp && item.timeStamp <= _this.searchEndTimeStamp)

          if (!_this.actObjs[i].posInfo.length) {
            _this.actObjs.splice(i, 1);
          }
          _this.actObjs[i].color = _this.colorArr[i % 10];
          this.$set(_this.actObjs[i], 'isHide', _this.actObjs[i].isHide);
        }
        console.log(new Date(), _this.actObjs);

        var keyArr = Object.keys(_this.actObj);
        for (var i = 0; i < keyArr.length; i++) {
          _this.actObj[keyArr[i]].forEach(dot => {
            dot.color = _this.colorArr[i % 10];
          })
        }
        console.log(new Date().getTime(), _this.actObj)
        _this.drawTracePath()
      })
    }
  }
}
</script>

<style lang="scss">
.trace-plot {
  .filter-bar {
    width: 960px;
    margin: 20px auto;
    display: flex;
    align-items: center;
    flex-direction: end;
    .btn-search {
      margin-left: 20px;
    }
  }
  .check-target {
    padding: 30px;
    box-sizing: border-box;
    overflow: auto;
    background: #f1f4f9;
    float: left;
    .target-list {
      display: flex;
      flex-wrap: wrap;
      justify-content: space-between;
      width: 300px;

      .list-item {
        width: 80px;
        height: 80px;
        margin-bottom: 10px;
        border: 3px solid yellow;
        border-radius: 50%;
        img {
          width: 100%;
          height: 100%;
          display: block;
          border-radius: 50%;
        }
        &.deact {
          opacity: 0.5;
          background: rgba(0, 0, 0, 0.3);
        }
      }
    }
  }
  canvas {
    //background: lightsteelblue;
    background-repeat: no-repeat;
  }
}
</style>